@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altoslib_4;
+package org.altusmetrum.altoslib_5;
 
 import java.io.*;
 
@@ -52,7 +52,7 @@ public class AltosDebug {
 	public static final byte GET_CHIP_ID =		0x68;
 
 
-	AltosLink	link;
+	private AltosLink	link;
 
 	boolean	debug_mode;
 
@@ -112,7 +112,11 @@ public class AltosDebug {
 		int i = 0;
 		int start = 0;
 		while (i < length) {
-			String	line = link.get_reply().trim();
+			String	line = link.get_reply();
+			if (line == null)
+				throw new IOException("No reply");
+
+			line = line.trim();
 			if (!AltosLib.ishex(line) || line.length() % 2 != 0)
 				throw new IOException(
 					String.format
@@ -277,4 +281,4 @@ public class AltosDebug {
 	public AltosDebug (AltosLink link) {
 		this.link = link;
 	}
-}
\ No newline at end of file
+}
