@@ -22,8 +22,8 @@ import java.awt.event.*;
 import javax.swing.*;
 import java.io.*;
 import java.util.concurrent.*;
-import org.altusmetrum.altoslib_4.*;
-import org.altusmetrum.altosuilib_2.*;
+import org.altusmetrum.altoslib_5.*;
+import org.altusmetrum.altosuilib_3.*;
 
 public class AltosUI extends AltosUIFrame {
 	public AltosVoice voice = new AltosVoice();
@@ -99,10 +99,25 @@ public class AltosUI extends AltosUIFrame {
 		return b;
 	}
 
+	/* OSXAdapter interfaces */
+	public void macosx_file_handler(String path) {
+		process_graph(new File(path));
+	}
+
+	public void macosx_quit_handler() {
+		System.exit(0);
+	}
+
+	public void macosx_preferences_handler() {
+		ConfigureAltosUI();
+	}
+
 	public AltosUI() {
 
 		load_library(null);
 
+		register_for_macosx_events();
+
 		AltosUIPreferences.set_component(this);
 
 		pane = getContentPane();
