@@ -15,14 +15,14 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altosuilib_2;
+package org.altusmetrum.altosuilib_3;
 
 import java.io.*;
 import java.util.ArrayList;
 
 import java.awt.*;
 import javax.swing.*;
-import org.altusmetrum.altoslib_4.*;
+import org.altusmetrum.altoslib_5.*;
 
 import org.jfree.ui.*;
 import org.jfree.chart.*;
@@ -48,7 +48,11 @@ public class AltosUIAxis extends NumberAxis {
 	public final static int axis_default = axis_include_zero;
 
 	public void set_units() {
-		setLabel(String.format("%s (%s)", label, units.show_units()));
+		String u = units.show_units();
+		if (u != null)
+			setLabel(String.format("%s (%s)", label, u));
+		else
+			setLabel(label);
 	}
 
 	public void set_enable(boolean enable) {
