@@ -15,7 +15,7 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
  */
 
-package org.altusmetrum.altosuilib_2;
+package org.altusmetrum.altosuilib_3;
 
 import java.awt.*;
 import java.awt.event.*;
@@ -223,6 +223,31 @@ public class AltosUIConfigure
 		row++;
 	}
 
+	static final Integer map_caches[] = { 9, 25, 100 };
+
+	public void add_map_cache() {
+		pane.add(new JLabel("Map Cache Size"), constraints(0, 1));
+
+		final JComboBox<Integer> map_cache = new JComboBox<Integer>(map_caches);
+
+		map_cache.setEditable(true);
+		map_cache.addActionListener(new ActionListener() {
+				public void actionPerformed(ActionEvent e) {
+					try {
+						int	size = (Integer) (map_cache.getSelectedItem());
+
+						AltosUIPreferences.set_map_cache(size);
+					} catch (ClassCastException ce) {
+						map_cache.setSelectedItem(new Integer(AltosUIPreferences.map_cache()));
+					}
+				}
+			});
+
+		map_cache.setSelectedItem (new Integer(AltosUIPreferences.map_cache()));
+		pane.add(map_cache, constraints(1, 2, GridBagConstraints.BOTH));
+		row++;
+	}
+
 	public void add_bluetooth() {
 	}
 
@@ -257,7 +282,9 @@ public class AltosUIConfigure
 		add_font_size();
 		add_look_and_feel();
 		add_position();
-		add_bluetooth();
+		add_map_cache();
+		if (AltosUILib.has_bluetooth)
+			add_bluetooth();
 		add_frequencies();
 
 		/* And a close button at the bottom */
