@@ -26,16 +26,16 @@ struct ao_event {
 	uint8_t		type;
 	uint8_t		unit;
 	uint16_t	tick;
-	uint32_t	value;
+	int32_t		value;
 };
 
-uint8_t
+void
 ao_event_get(struct ao_event *ev);
 
 void
-ao_event_put_isr(uint8_t type, uint8_t unit, uint32_t value);
+ao_event_put_isr(uint8_t type, uint8_t unit, int32_t value);
 
 void
-ao_event_put(uint8_t type, uint8_t unit, uint32_t value);
+ao_event_put(uint8_t type, uint8_t unit, int32_t value);
 
 #endif /* _AO_EVENT_H_ */
